/* xsoldier, a shoot 'em up game with "not shooting" bonus
 * Copyright (C) 1997 Yuusuke HASHIMOTO <s945750@educ.info.kanagawa-u.ac.jp>
 * Copyright (C) 2002 Oohara Yuuma  <oohara@libra.interq.or.jp>
 *
 * This is a copyleft program.  See the file LICENSE for details.
 */
/* $Id: ending.h,v 1.2 2002/04/11 23:45:20 oohara Exp $ */

#if !defined _ENDING_H_
#define _ENDING_H_

extern void Ending(void);

#endif
