/* xsoldier, a shoot 'em up game with "not shooting" bonus
 * Copyright (C) 1997 Yuusuke HASHIMOTO <s945750@educ.info.kanagawa-u.ac.jp>
 * Copyright (C) 2002 Oohara Yuuma  <oohara@libra.interq.or.jp>
 *
 * This is a copyleft program.  See the file LICENSE for details.
 */
/* $Id: version.h,v 1.10 2011/08/12 15:17:14 oohara Exp $ */

#include <config.h>
#if !defined _VERSION_H_
#define _VERSION_H_

/* VERSION is defined in config.h because we use automake */

#define UPDATE "13 Aug 2011"

#endif
