/* xsoldier, a shoot 'em up game with "not shooting" bonus
 * Copyright (C) 1997 Yuusuke HASHIMOTO <s945750@educ.info.kanagawa-u.ac.jp>
 * Copyright (C) 2002 Oohara Yuuma  <oohara@libra.interq.or.jp>
 *
 * This is a copyleft program.  See the file LICENSE for details.
 */
/* $Id: joystick.h,v 1.3 2011/08/10 14:25:42 oohara Exp $ */

#if !defined _JOYSTICK_H_
#define _JOYSTICK_H_

extern void initJS(void);
extern void readJS(void);

#endif /* not _JOYSTICK_H_ */
